package org.joda.money;

import java.math.BigDecimal;
import java.util.Arrays;

import org.testng.annotations.Test;

import junit.framework.TestCase;

public class TestBigMoney32 extends TestCase {

	private static final CurrencyUnit GBP = CurrencyUnit.of("GBP");
	private static final CurrencyUnit EUR = CurrencyUnit.of("EUR");
	private static final CurrencyUnit USD = CurrencyUnit.of("USD");
	private static final CurrencyUnit JPY = CurrencyUnit.of("JPY");
	private static final BigDecimal BIGDEC_2_34 = new BigDecimal("2.34");
	private static final BigDecimal BIGDEC_2_345 = new BigDecimal("2.345");
	private static final BigDecimal BIGDEC_M5_78 = new BigDecimal("-5.78");

	private static final BigMoney GBP_0_00 = BigMoney.parse("GBP 0.00");
	private static final BigMoney GBP_1_23 = BigMoney.parse("GBP 1.23");
	private static final BigMoney GBP_2_33 = BigMoney.parse("GBP 2.33");
	private static final BigMoney GBP_2_34 = BigMoney.parse("GBP 2.34");
	private static final BigMoney GBP_2_35 = BigMoney.parse("GBP 2.35");
	private static final BigMoney GBP_2_36 = BigMoney.parse("GBP 2.36");
	private static final BigMoney GBP_5_78 = BigMoney.parse("GBP 5.78");
	private static final BigMoney GBP_M1_23 = BigMoney.parse("GBP -1.23");
	private static final BigMoney GBP_M5_78 = BigMoney.parse("GBP -5.78");
	private static final BigMoney GBP_INT_MAX_PLUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MAX_VALUE) + 1);
	private static final BigMoney GBP_INT_MIN_MINUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MIN_VALUE) - 1);
	private static final BigMoney GBP_INT_MAX_MAJOR_PLUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MAX_VALUE) + 1) * 100);
	private static final BigMoney GBP_INT_MIN_MAJOR_MINUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MIN_VALUE) - 1) * 100);
	private static final BigMoney GBP_LONG_MAX_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MIN_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MAX_MAJOR_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney GBP_LONG_MIN_MAJOR_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney JPY_423 = BigMoney.parse("JPY 423");
	private static final BigMoney USD_1_23 = BigMoney.parse("USD 1.23");
	private static final BigMoney USD_2_34 = BigMoney.parse("USD 2.34");
	private static final BigMoney USD_2_35 = BigMoney.parse("USD 2.35");
	private static final BigMoneyProvider BAD_PROVIDER = new BigMoneyProvider() {
		@Override
		public BigMoney toBigMoney() {
			return null; // shouldn't return null
		}
	};

	private static BigDecimal bd(String str) {
		return new BigDecimal(str);
	}

	// -----------------------------------------------------------------------
	// minus(Iterable)
	// -----------------------------------------------------------------------
	public void test_minus_Iterable_BigMoneyProvider() {
		Iterable<BigMoneyProvider> iterable = Arrays.<BigMoneyProvider> asList(GBP_2_33, GBP_1_23);
		BigMoney test = GBP_2_34.minus(iterable);
	}

	public void test_minus_Iterable_BigMoney() {
		Iterable<BigMoney> iterable = Arrays.<BigMoney> asList(GBP_2_33, GBP_1_23);
		BigMoney test = GBP_2_34.minus(iterable);
	}

	public void test_minus_Iterable_Money() {
		Iterable<Money> iterable = Arrays.<Money> asList(GBP_2_33.toMoney(), GBP_1_23.toMoney());
		BigMoney test = GBP_2_34.minus(iterable);
	}

	public void test_minus_Iterable_Mixed() {
		Iterable<BigMoneyProvider> iterable = Arrays.<BigMoneyProvider> asList(GBP_2_33.toMoney(),
				new BigMoneyProvider() {
					@Override
					public BigMoney toBigMoney() {
						return GBP_1_23;
					}
				});
		BigMoney test = GBP_2_34.minus(iterable);
	}

	public void test_minus_Iterable_zero() {
		Iterable<BigMoneyProvider> iterable = Arrays.<BigMoneyProvider> asList(GBP_0_00);
		BigMoney test = GBP_2_34.minus(iterable);
	}

	public void test_minus_Iterable_currencyMismatch() {
		Iterable<BigMoneyProvider> iterable = Arrays.<BigMoneyProvider> asList(GBP_2_33, JPY_423);
		GBP_M5_78.minus(iterable);
	}

	public void test_minus_Iterable_nullEntry() {
		Iterable<BigMoneyProvider> iterable = Arrays.<BigMoneyProvider> asList(GBP_2_33, null);
		GBP_M5_78.minus(iterable);
	}

	public void test_minus_Iterable_nullIterable() {
		GBP_M5_78.minus((Iterable<BigMoneyProvider>) null);
	}

	public void test_minus_Iterable_badProvider() {
		Iterable<BigMoneyProvider> iterable = Arrays.<BigMoneyProvider> asList(new BigMoneyProvider() {
			@Override
			public BigMoney toBigMoney() {
				return null;
			}
		});
		GBP_M5_78.minus(iterable);
	}
	
	public static void main(String[] args) {
		TestBigMoney32 t = new TestBigMoney32();
		t.test_minus_Iterable_BigMoneyProvider();
		t.test_minus_Iterable_BigMoney();
		t.test_minus_Iterable_Money();
		t.test_minus_Iterable_Mixed();
		t.test_minus_Iterable_zero();
	}

}
